@@ -46,7 +46,7 @@ from ngs_plumbing._libdna import byte_frombit2bytearray, byte_frombit2bytes
 from ngs_plumbing._libdna import slice_frombit2bytes
 from ngs_plumbing._libdna import bit2_frombytes, bit2_frombytearray, bit2_frombuffer
 from ngs_plumbing._libdna import bit2reverse_frombit2bytes, bit2complement_frombit2bytes, bit2reversecomplement_frombit2bytes
-from ngs_plumbing.string import sequentialfragments, sequentialfragments_iter, randomstring
+from ngs_plumbing.ngsp_string import sequentialfragments, sequentialfragments_iter, randomstring
 # A: 0b00
 # C: 0b01
 # G: 0b10
