@@ -18,7 +18,18 @@
 """ """
 from struct import unpack
 from ngs_plumbing.dna import PackedDNABytes
-from ngs_plumbing._libdna import slice_frombit2bytes, bit2_frombytes
+from ngs_plumbing._libdna import slice_frombit2bytes, bit2_frombytes, bit2_frombytearray
+
+def kmerbytearray_frombytes_iter(unpacked, K, step=1):
+    v = bytearray(unpacked)
+    for i in xrange(0, len(unpacked)-K+1, step):
+        yield v[i:(i+K)]
+
+def kmerbit2_frombytes_iter(unpacked, K, step=1):
+    """ Return an iterator of bit2-packed segments from unpacked DNA """
+    v = bytearray(unpacked)
+    for i in xrange(0, len(unpacked)-K+1, step):
+        yield bit2_frombytearray(v[i:(i+K)])
 
 def kmerbytes_frombit2_iter(packed, K, step=1):
     """
@@ -30,7 +41,7 @@ def kmerbytes_frombit2_iter(packed, K, step=1):
     - step: iterating step for the sliding window
     """
     #assert(isinstance(packed, PackedDNABytes))
-    for i in xrange(0, (len(packed)*4 - K), step):
+    for i in xrange(0, len(packed)*4-K+1, step):
         res = slice_frombit2bytes(packed, i, i+K) 
         yield res
 
@@ -41,7 +52,7 @@ def kmerbit2_frombit2_iter(packed, K, step=1):
     sliding over a sequence of 2-bit packed DNA
     """
     #assert(isinstance(packed, PackedDNABytes))
-    for i in xrange(0, (len(packed)*4 - K), step):
+    for i in xrange(0, len(packed)*4-K+1, step):
         #print('slice %i:%i from %i' %(i, i+K, len(packed)))
         res = slice_frombit2bytes(packed, i, i+K)
         #FIXME: lousy hack - the right function is still missing from the API
@@ -54,7 +65,7 @@ def poskmerbit2_frombit2_iter(packed, K, step=1):
     sliding over a sequence of 2-bit packed DNA
     """
     #assert(isinstance(packed, PackedDNABytes))
-    for i in xrange(0, (len(packed)*4 - K), step):
+    for i in xrange(0, len(packed)*4-K+1, step):
         #print('slice %i:%i from %i' %(i, i+K, len(packed)))
         res = slice_frombit2bytes(packed, i, i+K)
         #FIXME: lousy hack - the right function is still missing from the API
