@@ -32,15 +32,17 @@ from ngs_plumbing._libxsq import colourqual_frombuffer, basequal_frombuffer, byt
 ColourQual = namedtuple('ColourQual', 'colour qual')
 
 _ACGT = bytes('ACGT')
-FRAGMENT_NAMES = ('F3', 'F5-DNA', 'F5-RNA', 'R3', 'R5')
+FRAGMENT_NAMES = ('F3', 'F5-DNA', 'F5-RNA', 'F5-BC', 'F5-2', 'R3', 'R5')
 # F3: Forward strand. starts with a T
 # F5/R3: Used to prime both F5-BC read and R3 or barcode. starts with a G
 #
 FRAGMENT_START = {'F3': b'T',
                   'F5-DNA': b'G',
                   'F5-RNA': b'G', # to be verified
+                  'F5-BC': None,
+                  'F5-2': None,
                   'R3': b'G',
-                  'R5': None # I don' know
+                  'R5': None # I don't know
 }
 
 _pack_installdir = os.path.abspath(os.path.dirname(__file__))
@@ -58,7 +60,6 @@ class FragmentError(KeyError):
 class InvalidXSQError(ValueError):
     pass
 
-
         
 class XSQFile(h5py.File):
     """
@@ -218,6 +219,7 @@ def iter_valuearrays(group, fragment, what='BaseCallQV'):
     for tile_n in group.keys():
         tile = group[tile_n]
         a = tile_valuearray(tile, fragment, what)
+        tile_n = bytes(tile_n)
         yield (tile_n, a)
 
 def iter_reads(group, fragment, what='BaseCallQV'):
@@ -329,7 +331,6 @@ class XSQInfo(OrderedDict):
                 continue
             try:
                 tmp = getattr(mtd, at)
-                #import pdb; pdb.set_trace()
                 value = str(fun(tmp))
             except AttributeError:
                 value = 'Error'
@@ -366,14 +367,12 @@ class XSQTagDetails(h5py._hl.group.Group):
     
 
 def seqid_template(f, flowcell, tile):
-
     try:
         serial = f.run_metadata.attrs['InstrumentSerial'][0]
-    except KeyError, ke:
+    except (AttributeError, KeyError), ke:
         serial = 'UnknownSerial'
-
         
-    seq_id = '@%s:%s:%s' %(serial,flowcell, tile) + ':%i:%i#0'
+    seq_id = '@%s:%s:%s' %(serial, flowcell, tile) + ':%i:%i#0'
     return seq_id
 
 def seqid_template_csfasta(flowcell, fragment):
@@ -406,8 +405,13 @@ def make_fastq(lib, fragment, buf, f, flowcell, numbase = None,
         xyloc = lib[tile_n]['Fragments'][u'yxLocation'][:]
 
         read_count += xyloc.shape[0]
-        #for i, x in enumerate(iter_fastq_reads(lib, fragment, numbase)):
-        for row_i in _progress_bar_iter(xrange(a.shape[0]), suffix = ' tile: %i' %(tile_i+1)): 
+                
+        ms = long(math.ceil((tile_i+1) * 1.0 * 20 / len(lib)))
+        out.write('\r|%s%s|tile %i' %('\xe2\x96\x88' * min(20, ms), 
+                                      ' ' * (20 - ms), tile_i+1))
+        out.flush()
+
+        for row_i in xrange(a.shape[0]):
             dna, qual = basequal_frombuffer(a[row_i])
             qual = bytearray_phredtoascii(qual)
             #yield DnaQual(dna, qual)
@@ -418,12 +422,7 @@ def make_fastq(lib, fragment, buf, f, flowcell, numbase = None,
                 n_perfect += inc
             dna_b = bytes(dna)
             x,y = xyloc[row_i]
-            buf.write(seq_id %(x,y))
-            buf.write('\n')
-            buf.write(dna_b)
-            buf.write('\n+\n')
-            buf.write(qual)
-            buf.write('\n')
+            buf.writelines((seq_id %(x,y), '\n', dna_b, '\n+\n', qual, '\n'))
 
         #numbase = None
 
@@ -436,6 +435,25 @@ def make_fastq(lib, fragment, buf, f, flowcell, numbase = None,
     out.flush()
 
 
+def _xsqvalues_to_csfasta(xsqvalues, xyloc, buf_cs, buf_qual, seq_id, prefix):
+    if xsqvalues.shape[1] == 0:
+        sys.stderr.write('Reads of length 0 (!?)\n')
+        sys.stderr.flush()
+        return
+    bases_i = range(xsqvalues.shape[1]-1)
+    for row_i in xrange(xsqvalues.shape[0]): 
+        seq, qual = colourqual_frombuffer(xsqvalues[row_i])
+        # turn integers to their ASCII representations
+        seq = bytearray_addint(seq, 48)
+        x,y = xyloc[row_i]
+        this_seq_id = seq_id % (x,y)
+        buf_cs.writelines((this_seq_id, '\n', prefix, seq, '\n'))
+        buf_qual.writelines((this_seq_id, '\n'))
+        for q_i in bases_i:
+            buf_qual.write('%i ' % qual[q_i])
+        buf_qual.writelines((str(qual[len(bases_i)]), '\n'))
+
+
 def make_csfasta(lib, fragment, buf_cs, buf_qual, f, flowcell, 
                  numbase = None, progress_mark = 50000, out=sys.stdout):
     read_count = long(0)
@@ -449,29 +467,27 @@ def make_csfasta(lib, fragment, buf_cs, buf_qual, f, flowcell,
         xyloc = lib[tile_n]['Fragments'][u'yxLocation'][:]
         read_count += xyloc.shape[0]
         prefix = FRAGMENT_START[fragment]
-        for row_i in _progress_bar_iter(xrange(a.shape[0]), 
-                                        suffix = ' tile: %i'%(tile_i+1)): 
-            seq, qual = colourqual_frombuffer(a[row_i])
-            # turn integers to their ASCII representations
-            seq = bytearray_addint(seq, 48)
-            
-            x,y = xyloc[row_i]
-            buf_cs.write(seq_id %(x,y))
-            buf_cs.write('\n')
-            buf_cs.write(prefix)
-            buf_cs.write(seq)
-            buf_cs.write('\n')
 
-            buf_qual.write(seq_id %(x,y))
-            buf_qual.write('\n')
-            buf_qual.write(' '.join( str(q) for q in qual))
-            buf_qual.write('\n')
+        ms = long(math.ceil((tile_i + 1) * 1.0 * 20 / len(lib)))
+        out.write('\r|%s%s|tile %i' %('\xe2\x96\x88' * min(20, ms), 
+                                      ' ' * (20 - ms), tile_i + 1))
+        out.flush()
+
+        _xsqvalues_to_csfasta(a, xyloc, buf_cs, buf_qual, seq_id, prefix)
 
     if os.isatty(out.fileno()):
         out.write('\n')
     out.write("%i reads (%.3fGb).\n" %(read_count, read_count*numbase / 1E9))
     out.flush()
 
+def _xsqvalues_to_csfastq(xsqvalues, xyloc, buf, seq_id, prefix):
+    for row_i in xrange(xsqvalues.shape[0]):
+        seq, qual = colourqual_frombuffer(xsqvalues[row_i])
+        seq = bytearray_addint(seq, 48)
+        qual = bytearray_phredtoascii(qual)
+        x,y = xyloc[row_i]
+        buf.writelines((seq_id %(x,y), '\n', prefix, seq, '\n+\n', qual, '\n'))
+        
 def make_csfastq(lib, fragment, buf, f, flowcell, 
                  numbase = None, progress_mark = 50000, out=sys.stdout):
     read_count = long(0)
@@ -485,19 +501,13 @@ def make_csfastq(lib, fragment, buf, f, flowcell,
         xyloc = lib[tile_n]['Fragments'][u'yxLocation'][:]
         read_count += xyloc.shape[0]
         prefix = FRAGMENT_START[fragment]
-        for row_i in _progress_bar_iter(xrange(a.shape[0]), 
-                                        suffix = ' tile: %i'%(tile_i+1)): 
-            seq, qual = colourqual_frombuffer(a[row_i])
-            seq = bytearray_addint(seq, 48)
-            
-            x,y = xyloc[row_i]
-            buf.write(seq_id %(x,y))
-            buf.write('\n')
-            buf.write(prefix)
-            buf.write(seq)
-            buf.write('\n+\n')
-            buf.write(' '.join( str(q) for q in qual))
-            buf.write('\n')
+
+        ms = long(math.ceil((tile_i+1) * 1.0 * 20 / len(lib)))
+        out.write('\r|%s%s|tile %i' %('\xe2\x96\x88' * min(20, ms), 
+                                      ' ' * (20 - ms), tile_i + 1))
+        out.flush()
+
+        _xsqvalues_to_csfastq(a, xyloc, buf, seq_id, prefix)
 
     if os.isatty(out.fileno()):
         out.write('\n')
