@@ -35,7 +35,12 @@ xsq = ngs_plumbing.xsq
 
 def main():
     parser = argparse.ArgumentParser(
-        description = 'Convert XSQ files into less exotic (albeit sometimes also less efficient) formats.')
+        description = '''Convert XSQ files into less exotic (albeit sometimes also less efficient) formats.
+
+The overall idea is that XSQ files can be exported to two general types of files FASTA+QUAL or FASTQ,
+and the exact file format for a given type will be depend on whether color-space or base-space data
+are wanted. For example, the FASTQ general type would become CSFASTQ for color-space and
+FASTQ for base-space.''')
     parser.add_argument('paths', metavar='XSQ_FILE', nargs='*',
                         help='XSQ files')
     parser.add_argument('-d', '--dir',
@@ -58,6 +63,15 @@ def main():
                         dest = 'want_info',
                         action = 'store_true',
                         help = 'Print information about the XSQ file')
+    parser.add_argument('--unassigned',
+                        dest = 'want_unassigned',
+                        action = 'store_true',
+                        help = 'Also extract data from unassigned barcodes.')
+    parser.add_argument('--buffer-size',
+                        dest = 'buffer_size',
+                        default = long(2E6),
+                        type = long,
+                        help = 'Buffer size in bytes. This can help with speed on some system or when files are accessed over NFS - default is set to 2000000 (2 Mb)')    
     parser.add_argument('-V', '--version',
                         action = 'version',
                         version = ngs_plumbing.__version__)
@@ -112,32 +126,38 @@ def main():
             if options.data_space == 'color':
                 xsq.automagic_csfasta(f, path_out = dest_dir,
                                       fragment = None, 
-                                      buf_size = long(2E6), 
-                                      out = sys.stdout)
+                                      buf_size = options.buffer_size, 
+                                      out = sys.stdout,
+                                      want_unassigned = options.want_unassigned)
             else:
-                print('--->')   
+                sys.stderr.write('Conversion to FASTA (base space) not yet implemented.\n')
+                sys.stderr.flush()
+                sys.exit(1)
         else:
             # FASTQ data
             if options.data_space == 'color':
                 xsq.automagic_fastq(f, path_out = dest_dir, 
                                     fragment = None, 
-                                    buf_size = long(2E6), 
+                                    buf_size = options.buffer_size, 
                                     out = sys.stdout,
                                     extension = 'csfq',
-                                    cs = True)
+                                    cs = True,
+                                    want_unassigned = options.want_unassigned)
             else:
                 # base space
                 if options.use_ecc:
                     try:
                         xsq.automagic_fastq(f, path_out = dest_dir, 
                                             fragment = None, 
-                                            buf_size = long(2E6), 
+                                            buf_size = options.buffer_size, 
                                             out = sys.stdout,
                                             cs = False)
                     except KeyError, ke:
                         print('\nError: No base calls for file "%s".' %f)
                 else:
-                    print('--->')
+                    sys.stderr.write('Conversion to FASTQ (base space) without ECC chemistry not yet implemented.\n')
+                    sys.stderr.flush()
+                    sys.exit(1)
 
     if (not options.data_space) or (not options.output_format):
         print('No output format or data space specified, so nothing was done. Goodbye.')
@@ -146,7 +166,9 @@ def htmlreport():
     import argparse
 
     parser = argparse.ArgumentParser(
-        description = 'Make an HTML report on XSQ files.')
+        description = '''
+Make an HTML report on the data quality in XSQ files.
+''')
     parser.add_argument('paths', metavar='XSQ_FILE', nargs='*',
                         help='XSQ files')
     parser.add_argument('-d', '--dir',
@@ -161,7 +183,7 @@ def htmlreport():
                         dest = 'sample_percent',
                         default = 0.05,
                         type = float,
-                        help = 'Percentage of the data to sample (a number between 0 and 1, default: 0.05, that is 5%%).')
+                        help = 'Percentage of the data to sample (a number between 0 and 1, default: 0.05, that is 5%%). An increased percentage is leading to an increased memory usage.')
     parser.add_argument('-t', '--data-type',
                         dest = 'data_type',
                         default = 'json',
